@@ -15,7 +15,7 @@ depends()
 #depends bootgen
 
 
-for XSA in ../inputs/XSA/*.xsa; do
+for XSA in ../inputs/*.xsa; do
 
     DESIGN=`basename "$XSA" | sed 's/[.].*//'`
 
