@@ -141,6 +141,40 @@ const char* get_xsa_parameter_value(xml* x, String name, bool debug)
 }
 
 
+int get_gpio_pin_from_name(xml* x, const char* pin_name)
+{
+  const char* pin_assignments = get_xsa_parameter_value(x, "PSU_MIO_TREE_SIGNALS");
+  if(!pin_assignments)
+    {
+      printf("Can't get MIO pin assignments.\n");
+      exit(20);
+    }
+
+  //printf("pin assignments are %s\n", (const char*)pin_assignments);
+
+  int pin_count = 0;
+  int i=0;
+  while(pin_assignments[i])
+    {
+      int match = stringMatchToLengthOfFirst(pin_name, &pin_assignments[i]);
+      if(match)
+	{
+	  //printf("Pin \"%s\" is assigned to MIO_%d\n", pin_name, pin_count);
+	  return pin_count;
+	}
+      
+      while(pin_assignments[i] && pin_assignments[i]!='#')
+	i++;
+
+      if(pin_assignments[i])
+	i++;
+      pin_count++;
+    }
+  
+  //printf("Can't find pin assignment for MIO pin \"%s\"\n", pin_name);
+  //exit(20);
+  return -1;
+}
 
 
 void get_addresses_from_map(const char* PSU__PROTECTION__SLAVES, const char* key, unsigned long long& start, unsigned long long& end, bool& enabled)
@@ -386,6 +420,18 @@ void write_device_tree(xml* x, writestream& w)
     "};\n"
     "\n";
 
+//
+// Don't normally define the GEM in the device tree, since
+// it can't be fully specified anyway.  The PHY address is
+// set externally to the ZYNQ, in typical configurations,
+// and isn't included in the Vivado ZYNQ block diagram
+// configuration.  So it's not in the HWH file inside the
+// XSA file.  We have no way to know the PHY address here.
+// So it must be added manually anyway.  Thus do the
+// entire GEM configuration manually by steps external
+// to this program
+//
+#ifdef DEFINE_THE_GEM
   srand48((long int) time(NULL));
 
   for(int i=0; i<=3; i++)
@@ -396,6 +442,9 @@ void write_device_tree(xml* x, writestream& w)
 	{
 	  //printf("Ethernet gem%d is enabled.\n", i);
 
+	  sprintf(tmp, "PSU__ENET%d__GRP_MDIO__ENABLE", i);
+	  bool uses_mdio = one.equals(get_xsa_parameter_value(x, tmp));
+	  
 	  w <<
 	    "\n"
 	    "&gem" << i <<  " {\n";
@@ -416,7 +465,39 @@ void write_device_tree(xml* x, writestream& w)
 	    "\n"
 	    "   phy-mode = \"rgmii-id\";\n"
 	    "   status = \"okay\";\n"
-	    "   xlnx,ptp-enet-clock = <0x0>;\n"
+	    "   xlnx,ptp-enet-clock = <0x0>;\n";
+
+
+	  //
+	  // ZCU111 uses phy reg c.  RFSoC4x2 uses phy reg f
+	  // These values come from the PHY chip (83867) external
+	  // to the ZYNQ.  They are set by strapping resistors.
+	  // There is no entry in the Vivado ZYNQ block design
+	  // configuration to set this, so it must be set in the
+	  // device tree by some other method.
+	  //
+	  
+	  if(uses_mdio)
+	    {
+	      w <<
+		"   phy-handle = <&phy0>;\n"
+		"   mdio: mdio {\n"
+		"      #address-cells = <1>;\n"
+		"      #size-cells = <0>;\n"
+		"      phy0: ethernet-phy@f {\n"
+		"         #phy-cells = <1>;\n"
+		"         compatible = \"ethernet-phy-id2000.a231\";\n"
+		"         reg = <0xf>;\n"
+		"         ti,rx-internal-delay = <0x8>;\n"
+		"         ti,tx-internal-delay = <0xa>;\n"
+		"         ti,fifo-depth = <0x1>;\n"
+		"         ti,dp83867-rxctrl-strap-quirk;\n"
+		"         //reset-gpios = <&tca6416_u22 6 GPIO_ACTIVE_LOW>;\n"
+		"      };\n"
+		"   };\n";
+	    }
+	  
+	  w <<
 	    "};\n"
 	    "\n";
 
@@ -424,7 +505,8 @@ void write_device_tree(xml* x, writestream& w)
 	}
       
     }
-
+#endif // DEFINE_THE_GEM
+  
 
   //
   // No known source for this in the XSA's HWH file:
@@ -577,12 +659,6 @@ void write_device_tree(xml* x, writestream& w)
       w <<
 	"&zynqmp_dpsub {\n";
 
-      //
-      // Not sure whether the PHYs and lanes are confused here.
-      // The ref_clks seem to be scrambed by the fsbl, so their
-      // numbers in the XSA may be wrong!  Working on something
-      // to patch over this.
-      //
       //
       // Note that the phys are <&psgtr  GTR_LANE  PHY_TYPE_DP  PHY_INSTANCE  REFCLK_NUMBER>
       //
@@ -751,6 +827,39 @@ void write_device_tree(xml* x, writestream& w)
 	      printf("Can't get SD card %d clock frequency.\n", i);
 	      exit(20);
 	    }
+
+	  const char* pin_name = (i==0) ? "sdio0_clk_out" : "sdio1_clk_out";
+
+	  int pin_number = get_gpio_pin_from_name(x, pin_name);
+
+	  int bank = 4;
+	  
+	  if(pin_number<0)
+	    {
+	      printf("WARNING:  Can't get pin number for sdio%d\n", i);
+	    }
+	  else if(pin_number<=25)
+	    {
+	      bank = 0;
+	    }
+	  else if(pin_number<=51)
+	    {
+	      bank = 1;
+	    }
+	  else if(pin_number<=51)
+	    {
+	      bank = 2;
+	    }
+	  else
+	    {
+	      bank = 3;
+	    }
+
+	  const char* wp_name = (i==0) ? "sdio0_wp" : "sdio1_wp";
+  
+	  int wp_pin = get_gpio_pin_from_name(x, wp_name);
+
+	  const char* disable_wp = (wp_pin<0) ? "   disable-wp;\n" : "";
 	  
 	  long freq_Hz = atof(freq_MHz) * 1000000l;
 
@@ -760,8 +869,9 @@ void write_device_tree(xml* x, writestream& w)
 	    "&sdhci" << i << " {\n"
 	    "   clock-frequency = <" << freq_Hz << ">;\n"
 	    "   status = \"okay\";\n"
-	    "   xlnx,mio-bank = <0x0>;\n"
-	    "   no-1-8-v;\n"
+	    "   xlnx,mio-bank = <" << bank << ">;\n"
+	    "   no-1-8-v;\n" <<
+	    disable_wp <<
 	    "   };\n";
 	}
     }
