@@ -1,7 +1,7 @@
 /* SPDX-License-Identifier: GPL-2.0 */
 /*
  * Copyright (C) 2019-2022 Xilinx, Inc.
- * Copyright (C) 2022 Advanced Micro Devices, Inc. All Rights Reserved.
+ * Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All Rights Reserved.
  */
 
 #ifndef _DT_BINDINGS_VERSAL_POWER_H
@@ -14,6 +14,7 @@
 #define PM_DEV_SPI_1				(0x1822401cU)
 #define PM_DEV_I2C_0				(0x1822401dU)
 #define PM_DEV_I2C_1				(0x1822401eU)
+#define PM_DEV_I2C_PMC                          (0x1822402dU)
 #define PM_DEV_CAN_FD_0				(0x1822401fU)
 #define PM_DEV_CAN_FD_1				(0x18224020U)
 #define PM_DEV_UART_0				(0x18224021U)
@@ -23,6 +24,7 @@
 #define PM_DEV_TTC_1				(0x18224025U)
 #define PM_DEV_TTC_2				(0x18224026U)
 #define PM_DEV_TTC_3				(0x18224027U)
+#define PM_DEV_SWDT_LPD				(0x18224028U)
 #define PM_DEV_SWDT_FPD				(0x18224029U)
 #define PM_DEV_OSPI				(0x1822402aU)
 #define PM_DEV_QSPI				(0x1822402bU)
